#------------------------------------------------------------------------------
# Copyright (c) 2006 by Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
""" Manages type extensions, including factories to generate adapters, and
    hooks for methods and functions. Part of the AppTools project of the
    Enthought Tool Suite.
"""
from .api import *
