# Copyright (c) 2007-2011 by Enthought, Inc.
# All rights reserved.
""" Supports limiting access to parts of an application to authorised users.
    Part of the AppTools project of the Enthought Tool Suite.
"""
